# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/kpimon/kpimon.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import betterproto
import grpclib


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Object(betterproto.Message):
    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    attributes: Dict[str, int] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_UINT64
    )

    def __post_init__(self) -> None:
        super().__post_init__()


class KpimonStub(betterproto.ServiceStub):
    async def get_num_active_u_es(self, *, id: str = "") -> "GetResponse":

        request = GetRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.kpimon.Kpimon/GetNumActiveUEs", request, GetResponse
        )
